@@ -1,7 +1,7 @@
 /*
  * connection-manager.c - HazeConnectionManager source
  * Copyright (C) 2007 Will Thompson
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2009 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,33 +40,23 @@ G_DEFINE_TYPE(HazeConnectionManager,
  * hyphens rather than underscores for consistency.
  */
 static HazeProtocolInfo known_protocol_info[] = {
-    { "aim",        "prpl-aim",         NULL,
-        "always_use_rv_proxy:always-use-rv-proxy" },
+    { "aim",        "prpl-aim",         NULL, "" },
+    /* Seriously. */
+    { "facebook",   "prpl-bigbrownchunx-facebookim", NULL, "" },
     { "gadugadu",   "prpl-gg",          NULL, "" },
     { "groupwise",  "prpl-novell",      NULL, "" },
     { "irc",        "prpl-irc",         NULL, "encoding:charset" },
-    { "icq",        "prpl-icq",         NULL,
-        "encoding:charset,always_use_rv_proxy:always-use-rv-proxy" },
+    { "icq",        "prpl-icq",         NULL, "encoding:charset" },
     { "jabber",     "prpl-jabber",      NULL,
-        "connect_server:server,old_ssl:old-ssl,require_tls:require-encryption,"
-        "auth_plain_in_clear:auth-plain-in-clear,ft_proxies:ft-proxies" },
+        "connect_server:server,require_tls:require-encryption" },
     { "local-xmpp", "prpl-bonjour",     NULL,
         "first:first-name,last:last-name" },
-    { "msn",        "prpl-msn",         NULL,
-        "http_method:http-method,http_method_server:http-method-server,"
-        "custom_smileys:custom-smileys" },
-    { "qq",         "prpl-qq",          NULL, "use_tcp:use-tcp" },
-    { "sametime",   "prpl-meanwhile",   NULL,
-        "force_login:force-login,fake_client_id:fake-client-id" },
-    { "yahoo",      "prpl-yahoo",       NULL,
-        "local_charset:charset,xfer_host:xfer-host,xferjp_host:xferjp-host,"
-        "xfer_port:xfer-port,room_list_locale:room-list-locale,"
-        "ignore_invites:ignore-invites" },
-    { "zephyr",     "prpl-zephyr",      NULL,
-        "encoding:charset,use_tzc:use-tzc,tzc_command:tzc-command,"
-        "write_anyone:write-anyone,write_zsubs:write-zsubs,"
-        "read_anyone:read-anyone,read_zsubs:read-zsubs,"
-        "exposure_level:exposure-level" },
+    { "msn",        "prpl-msn",         NULL, "" },
+    { "qq",         "prpl-qq",          NULL, "" },
+    { "sametime",   "prpl-meanwhile",   NULL, "" },
+    { "yahoo",      "prpl-yahoo",       NULL, "local_charset:charset" },
+    { "yahoojp",    "prpl-yahoojp",     NULL, "local_charset:charset" },
+    { "zephyr",     "prpl-zephyr",      NULL, "encoding:charset" },
     { NULL,         NULL,               NULL, "" }
 };
 
@@ -104,6 +94,56 @@ _haze_cm_set_param (const TpCMParamSpec *paramspec,
     g_hash_table_insert (params, prpl_param_name, value_copy);
 }
 
+static gboolean
+_param_filter_no_blanks (const TpCMParamSpec *paramspec,
+                         GValue *value,
+                         GError **error)
+{
+    const gchar *str = g_value_get_string (value);
+
+    if (*str == '\0')
+    {
+        g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+            "Account parameter '%s' must not be empty",
+            paramspec->name);
+        return FALSE;
+    }
+
+    if (strstr (str, " ") != NULL)
+    {
+        g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+            "Account parameter '%s' may not contain spaces",
+            paramspec->name);
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+/* Checks whether the supplied string equals one of those in the GList
+ * paramspec->filter_data.
+ */
+static gboolean
+_param_filter_string_list (const TpCMParamSpec *paramspec,
+                           GValue *value,
+                           GError **error)
+{
+    const gchar *str = g_value_get_string (value);
+    const GList *valid_values = paramspec->filter_data;
+
+    for (; valid_values != NULL; valid_values = valid_values->next)
+    {
+        const gchar *valid = valid_values->data;
+
+        if (!tp_strdiff (valid, str))
+            return TRUE;
+    }
+
+    g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+        "'%s' is not a valid value for parameter '%s'", str, paramspec->name);
+    return FALSE;
+}
+
 /* Populates a TpCMParamSpec from a PurpleAccountOption, possibly renaming the
  * parameter as specified in parameter_map.  paramspec is assumed to be zeroed out.
  * Returns TRUE on success, and FALSE if paramspec could not be populated (and
@@ -117,10 +157,23 @@ _translate_protocol_option (PurpleAccountOption *option,
 {
     const char *pref_name = purple_account_option_get_setting (option);
     PurplePrefType pref_type = purple_account_option_get_type (option);
-    gchar *name = g_hash_table_lookup (parameter_map, pref_name);
+    gchar *name = g_strdup (g_hash_table_lookup (parameter_map, pref_name));
+
     /* These strings are never free'd, but need to last until exit anyway.
      */
-    paramspec->name = g_strdup (name ? name : pref_name);
+    if (name == NULL)
+      name = g_strdup (pref_name);
+
+    if (g_str_has_prefix (name, "facebook_"))
+      {
+        gchar *tmp = g_strdup (name + strlen ("facebook_"));
+        g_free (name);
+        name = tmp;
+      }
+
+    g_strdelimit (name, "_", '-');
+    paramspec->name = name;
+
     paramspec->setter_data = option->pref_name;
     /* TODO: does libpurple ever require a parameter besides the username
      *       and possibly password?
@@ -166,9 +219,39 @@ _translate_protocol_option (PurpleAccountOption *option,
             else
                 def = purple_account_option_get_default_string (option);
 
-            if (def && *def)
+            if (def != NULL && *def != '\0')
             {
-                paramspec->def = g_strdup (def);
+                paramspec->def = def;
+                paramspec->flags |= TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT;
+            }
+            break;
+        }
+        case PURPLE_PREF_STRING_LIST:
+        {
+            const gchar *def;
+            const GList *option_tuples;
+            GList *valid_strings = NULL;
+            /* tuple->key is human-readable description, tuple->value is the
+             * value's ID and is secretly a (const char *).
+             */
+            const PurpleKeyValuePair *tuple;
+
+            paramspec->dtype = DBUS_TYPE_STRING_AS_STRING;
+            paramspec->gtype = G_TYPE_STRING;
+
+            option_tuples = purple_account_option_get_list (option);
+            for (; option_tuples != NULL; option_tuples = option_tuples->next)
+            {
+                tuple = option_tuples->data;
+                valid_strings = g_list_prepend (valid_strings, tuple->value);
+            }
+            paramspec->filter = _param_filter_string_list;
+            paramspec->filter_data = valid_strings;
+
+            def = purple_account_option_get_default_list_value (option);
+            if (def != NULL && *def != '\0')
+            {
+                paramspec->def = def;
                 paramspec->flags |= TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT;
             }
             break;
@@ -179,6 +262,9 @@ _translate_protocol_option (PurpleAccountOption *option,
             return FALSE;
     }
 
+    if (g_str_equal (paramspec->name, "server"))
+        paramspec->filter = _param_filter_no_blanks;
+
     return TRUE;
 }
 
@@ -313,32 +399,6 @@ get_protocols (HazeConnectionManagerClass *klass)
     return protocols;
 }
 
-HazeConnection *
-haze_connection_manager_get_haze_connection (HazeConnectionManager *self,
-                                             PurpleAccount *account)
-{
-    HazeConnection *hc;
-    GList *l = self->connections;
-
-    while (l != NULL) {
-        hc = l->data;
-        if(hc->account == account) {
-            return hc;
-        }
-    }
-
-    return NULL;
-}
-
-static void
-connection_shutdown_finished_cb (TpBaseConnection *conn,
-                                 gpointer data)
-{
-    HazeConnectionManager *self = HAZE_CONNECTION_MANAGER (data);
-
-    self->connections = g_list_remove(self->connections, conn);
-}
-
 static TpBaseConnection *
 _haze_connection_manager_new_connection (TpBaseConnectionManager *base,
                                          const gchar *proto,
@@ -357,11 +417,11 @@ _haze_connection_manager_new_connection (TpBaseConnectionManager *base,
                                          "parameters",      params,
                                          NULL);
 
-    cm->connections = g_list_prepend(cm->connections, conn);
-    g_signal_connect (conn, "shutdown-finished",
-                      G_CALLBACK (connection_shutdown_finished_cb),
-                      cm);
-
+    if (!haze_connection_create_account (conn, error))
+      {
+        g_object_unref (conn);
+        return FALSE;
+      }
     return (TpBaseConnection *) conn;
 }
 
@@ -463,13 +523,3 @@ haze_connection_manager_init (HazeConnectionManager *self)
 {
     DEBUG ("Initializing (HazeConnectionManager *)%p", self);
 }
-
-HazeConnectionManager *
-haze_connection_manager_get (void) {
-    static HazeConnectionManager *manager = NULL;
-    if (G_UNLIKELY(manager == NULL)) {
-        manager = g_object_new (HAZE_TYPE_CONNECTION_MANAGER, NULL);
-    }
-    g_assert (manager != NULL);
-    return manager;
-}
